#version 330 core
out vec4 fragColor;

in vec3 fragNormal;
in vec2 fragmentUV;
in vec3 FragPos;

uniform int color_check;
uniform vec3 color;

uniform samplerCube depthMap;

uniform vec3 lightPos;
uniform vec3 viewPos;
uniform vec3 lightColor;

uniform float far_plane;
uniform bool shadows;

struct Material {  
    float shininess;
	sampler2D texture_diffuse;
    sampler2D texture_specular;
}; 

uniform Material material;

// array of offset direction for sampling
vec3 gridSamplingDisk[20] = vec3[]
(
   vec3(1, 1,  1), vec3( 1, -1,  1), vec3(-1, -1,  1), vec3(-1, 1,  1), 
   vec3(1, 1, -1), vec3( 1, -1, -1), vec3(-1, -1, -1), vec3(-1, 1, -1),
   vec3(1, 1,  0), vec3( 1, -1,  0), vec3(-1, -1,  0), vec3(-1, 1,  0),
   vec3(1, 0,  1), vec3(-1,  0,  1), vec3( 1,  0, -1), vec3(-1, 0, -1),
   vec3(0, 1,  1), vec3( 0, -1,  1), vec3( 0, -1, -1), vec3( 0, 1, -1)
);

float ShadowCalculation(vec3 FragPos)
{
    // get vector between fragment position and light position
    vec3 fragToLight = FragPos - lightPos;

    float currentDepth = length(fragToLight);
    
    float shadow = 0.0;
    float bias = 0.15;
    int samples = 20;
    float viewDistance = length(viewPos - FragPos);
    float diskRadius = (1.0 + (viewDistance / far_plane)) / 25.0;
    for(int i = 0; i < samples; ++i)
    {
       float closestDepth = texture(depthMap, fragToLight + gridSamplingDisk[i] * diskRadius).r;
        closestDepth *= far_plane;   // undo mapping [0;1]
        if(currentDepth - bias > closestDepth)
            shadow += 1.0;
    }
    shadow /= float(samples);
        
    return shadow;
}

void main()
{           
    if (color_check == 1)
	{
	fragColor = vec4(color, 1);
	} 
	if (color_check == 0)
	{
    vec3 normal = normalize(fragNormal);
    // ambient
    vec3 ambient = lightColor * vec3(texture(material.texture_diffuse, fragmentUV));
    // diffuse
    vec3 lightDir = normalize(lightPos - FragPos);
    float diff = max(dot(lightDir, normal), 0.0);
    vec3 diffuse = diff * lightColor * vec3(texture(material.texture_diffuse, fragmentUV));
    // specular
    vec3 viewDir = normalize(viewPos - FragPos);
    vec3 reflectDir = reflect(-lightDir, normal);
    float spec = pow(max(dot(viewDir, reflectDir), 0.0), material.shininess);
    vec3 specular = spec * lightColor;    
    // calculate shadow 
	float shadow = shadows ? ShadowCalculation(FragPos) : 0.0;             
    vec3 lighting = (ambient + (1.0 - shadow) * (diffuse + specular)) * vec3(texture(material.texture_diffuse, fragmentUV));    

    fragColor = vec4(lighting, 1.0);
	}
}